-- hranova reprezentace stromu
create table atree
(id int primary key, -- identifikator uzlu
 parent_id int,      -- identifikator rodice
 name text);         -- hodnota uzlu

insert into atree (id, parent_id, name) values
(1, NULL, 'Alice'),
(2, 1, 'Bob'),
(3, 1, 'Chuck'),
(4, 1, 'David'),
(5, 2, 'Eman'),
(6, 2, 'Frank'),
(7, 4, 'Grace'),
(8, 4, 'Heidi'),
(9, 6, 'Ivan');


-- vraceni korenoveho uzlu
select t1.name from atree t1 where parent_id is null;

-- vraci prvni dve patra stromu
select t1.name n1, t2.name n2 
	from 	atree t1 
		left outer join atree t2 on (t2.parent_id = t1.id) 
	where 	t1.parent_id is null;

-- vraci prvni tri patra stromu
select t1.name n1, t2.name n2, t3.name n3 
	from 	atree t1 
		left outer join atree t2 on (t2.parent_id = t1.id) 
		left outer join atree t3 on (t3.parent_id = t2.id)
	where 	t1.parent_id is null;

-- vraci prvni ctyri patra stromu
select t1.name n1, t2.name n2, t3.name n3, t4.name n4 
	from 	atree t1 
		left outer join atree t2 on (t2.parent_id = t1.id) 
		left outer join atree t3 on (t3.parent_id = t2.id)
		left outer join atree t4 on (t4.parent_id = t3.id)
	where 	t1.parent_id is null;

-- konstrukt `with' (CTE) umoznuje vytvorit docasnou pojmenovanou tabulku,
-- kterou je mozne pouzit v dotazu
-- ukazkovy priklad pouziti:
with 
	foo as (select * from atree where name = 'Grace'),
	bar as (select * from atree where parent_id < 4)
	select * from foo union select * from bar;

-- konstrukt `with recursive' umoznuje vytvorit linearne rekurzivni dotaz
-- dotaz vrati vsechny nadrazene uzly uzlu s hodnotou 'Grace'
with recursive struct as (
	select id, parent_id, name from atree where name = 'Grace'
	union distinct 
	select t.id, t.parent_id, t.name  from atree t, struct where t.id = struct.parent_id
) select * from struct;

-- dotaz vrati vsechny nadrazene uzly uzlu s id = 7, pridava delku cesty
with recursive struct as (
	select id, parent_id, name, 1 as lvl from atree where id = 7 
	union distinct 
	select t.id, t.parent_id, t.name, lvl + 1 from atree t, struct where t.id = struct.parent_id
) select * from struct;
