-- nested set reprezentace stromu
create table tree
(label text primary key,              -- hodnota uzlu
 lt int not null unique deferrable,   -- levy index, deferrable umoznuje porusit omezeni v ramci jedne transakce
 rt int not null unique deferrable);  -- pravy index

create index tree_lt_idx on tree (lt);
create index tree_rt_idx on tree (rt);

insert into tree values
('Alice', 1, 12),
('Bob',   2, 3),
('Chuck', 4, 11),
('David', 5, 6),
('Eman',  7, 8),
('Frank', 9, 10);

-- koren
select * from tree where lt = 1;

-- listy
select * from tree where rt - lt = 1;
select * from tree where lt = rt - 1;

-- vypis vsech potomku
select 	r1.label as from_node, r2.label to_node
	from tree as r1, tree as r2
	where (r2.lt between r1.lt and r1.rt)
          and (r2.rt between r1.lt and r1.rt);

-- vypis vsech potomku; jednodussi verze
select 	r1.label as from_node, r2.label to_node
	from tree as r1, tree as r2
	where (r2.lt between r1.lt and r1.rt);

-- hloubka uzlu
select 	r2.label, count(r1.label) - 1 as node_level
	from tree as r1, tree as r2
	where r2.lt between r1.lt and r1.rt
	group by r2.label;

-- vyska stromu???

--
-- pomocne pohledy
--

-- hloubka uzlu
create view depth_of as
	select 	r2.label, count(r1.label) - 1 as node_level
		from tree as r1, tree as r2
		where r2.lt between r1.lt and r1.rt
		group by r2.label;

-- existence cesty z uzlu do uzlu 
create view from_to as
	select 	r1.label as from_node, r2.label as to_node
		from tree as r1, tree as r2
		where (r2.lt between r1.lt and r1.rt);

-- seznam delek vsech cest z uzlu do uzlu
create view path_len as
	select 	from_node, to_node, d2.node_level - d1.node_level as len
		from from_to, depth_of as d1, depth_of as d2
		where (from_to.from_node = d1.label)
                  and (from_to.to_node = d2.label);

-- primi potomci rodic - primy potomek
create view cover as
	select 
		r1.label as parent_node,
		r2.label as child_node,
		r2.lt as lt,
		r2.rt
	from 	tree as r1, tree as r2
	where 	r2.lt > r1.lt and r2.lt < r1.rt
		and not exists 
			(select * from tree as rm
			 where 	(rm.lt > r1.lt) and (rm.lt < r1.rt)
				and (r2.lt > rm.lt) and (r2.lt < rm.rt));

-- rekonstrukce stromu z nelistovych casti
select parent_node, min(lt) - 1 as lt, max(rt) + 1 as rt from cover group by parent_node;

-- nejlevejsi a nejpravejsi potemek
select 	r1.label as parent,
	r2.label as leftmost,
	r3.label as rightmost
	from tree as r1, tree as r2, tree as r3
	where (r2.lt = r1.lt + 1) and (r3.rt = r1.rt - 1);
	
-- vrat poradi potomka
select 	c1.parent_node, c1.child_node, count(c1.*) as nth
	from cover as c1, cover as c2
	where c1.parent_node = c2.parent_node and c1.lt >= c2.lt
	group by c1.parent_node, c1.child_node
